import { Locator, Page } from 'playwright';

const GLOBAL_FILTERS = [
  'Федеральные министерства и ведомства',
  'Правительство России и СССР',
  'Президент России и СССР',
  'Органы судебной власти РФ и СССР',
  'Органы законодательной власти России и СССР'
] as const;

const LOCAL_TITLE = 'Органы власти Республики Коми';
const LOCAL_FILTERS = [
  'Правительство Республики Коми',
  'Государственный Совет Республики Коми',
  'Глава Республики Коми',
  'Президиум Верховного Совета Республики Коми',
  'Президиум Верховного Совета Коми ССР',
  'Верховный Совет Коми АССР',
  'Верховный Совет Коми ССР',
  'Верховный Совет Республики Коми'
] as const;

export interface SearchOptions {
  textPrompt?: string;
  titlePrompt?: string;
  onlyActive?: boolean;
}

export async function executeSearch(page: Page, options: SearchOptions) {
  await page.getByRole('button', { name: 'Расширенный поиск' }).click();

  const clearButton = page.getByRole('link', { name: 'Очистить' });
  if ((await clearButton.count()) > 0) {
    await clearButton.first().click();
    await page.waitForTimeout(1000);
  }

  if (!!options.textPrompt) {
    await page
      .locator('label:has-text("Слова в тексте")')
      .locator('..')
      .locator('..')
      .locator('textarea')
      .fill(options.textPrompt);
  }

  if (!!options.titlePrompt) {
    await page
      .locator('label:has-text("Слова в названии")')
      .locator('..')
      .locator('..')
      .locator('textarea')
      .fill(options.titlePrompt);
  }

  if (options.onlyActive) {
    const status = page.getByText('Статус', { exact: true });
    const isStatusVisible = await status.isVisible().catch(() => false);
    if (!isStatusVisible) {
      await page.getByText('Дополнительные реквизиты документа').click();
    }
    await status.click();

    const includeActive = page.getByRole('cell', { name: 'Действующие', exact: true });
    if (!(await isItemChecked(includeActive))) {
      await includeActive.locator('img').nth(1).click();
    }
    await page.getByRole('button', { name: 'Выбрать', exact: true }).click();
  }

  await page.getByText('Орган / источник').click();

  const localItem = page.getByRole('cell', { name: LOCAL_FILTERS[0], exact: true });
  const isLocalExpanded = await localItem.isVisible().catch(() => false);
  if (!isLocalExpanded) {
    await page.getByRole('cell', { name: LOCAL_TITLE, exact: true }).click();
  }
  for (const filter of LOCAL_FILTERS) {
    const listItem = page.getByRole('cell', { name: filter, exact: true }).first();
    if (!(await isItemChecked(listItem))) {
      await listItem.locator('img').nth(2).click();
    }
  }

  for (const filter of GLOBAL_FILTERS) {
    const listItem = page.getByRole('cell', { name: filter, exact: true }).first();
    if (!(await isItemChecked(listItem))) {
      await listItem.locator('img').nth(1).click();
    }
  }

  await page.getByRole('button', { name: 'Выбрать', exact: true }).click();
  await page.getByRole('button', { name: 'Найти (Enter)' }).click();
}

// ========== INTERNALS ===========
async function isItemChecked(item: Locator) {
  const classAttr = await item.getAttribute('class');
  return !!classAttr && classAttr.includes('checked');
}
